(function() {
  'use strict';

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 5;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.results = {
      'practice trial': 0,
      'unanimous group': 0,
      'dissenter present': 0
    };
  };

  Trial.prototype.init = function () {
    window.onkeypress = this.startKeyListener.bind( this, 'begin' );
    $(document).ready(function () {
      this.createData();
      this.data[0].jq.show();
      this.data[1].jq.remove();

      if (!window.platform.mobile) {
        $('.text-container-1').keyup(function (e) {
          trial.startKeyListener('begin', e);
        });
        $('.text-container-2').keyup(function (e) {
          trial.startKeyListener('begin', e);
        });
      }
      if (window.opener.platform.os.family === 'iOS') {
        $('.experiment p').eq(0).focus();
        $('body').append($('<div class="ada-shadow-speech" aria-live="polite"></div>'));
        $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
        setTimeout(function () {
          $('#focus_elem').focus();
        }, 2000);
      }
      if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
        $('.only-desktop').remove();

        var buttonsStr = '';
        var startNum = this.i <= 1 ? 2 : 1;
        var endNum = this.i <= 1 ? 11 : 6;

        for (var i = startNum; i < endNum; i++) {
          buttonsStr += '<button id="button_' + i + '">' + i + '</button>';
        }

        $('.experiment').append($('<div class="button_container">' + buttonsStr + '</div>'));

        $('.button_container button').click(function () {
          var answer = $(this).html();
          trial.addAnswer(answer);
          if ( trial.i === 5 ) {
           trial.trialTextArray[trial.i].hide();
          }
          trial.showTrial();
        });
        $('.button_container').hide();

        $('#press_instruction').html('Press the space bar or <span class="visually-hidden">double</span> tap <br/>to begin the experiment.');
        $('.trial-text').html($('.trial-text').html().split('y/n').join('yes/no'));

        for (var i = 0; i < 5; i++) {
          setTimeout(function () {
            $('.experiment').focus();
          }, 100);
        }
        $('.experiment').bind('click', function () {
          trial.start('begin');
          $('.experiment').unbind('click');
        });
      }

      $('.close-button button').click(function (e) {
        e.preventDefault();
        window.opener.player.close_experiment();
      });
      $('.close-button button').keyup(function (e) {
        if (e.keyCode === 32) {
          e.preventDefault();
          if (window.platform.iOS) {
            window.focus();
            window.close();
          }
          else {
            window.opener.player.close_experiment();
          }
        }
      });

      setTimeout(function () {
        var $p = $('.common-inst');

        $('.experiment > p')
          .attr('aria-label', $p.text() + ' ' + this.data[0].jq.text() + ' ' + $('#press_instruction').text())
          .focus();
      }.bind(this), 0);
    }.bind(this));
  };

  Trial.prototype.removeExtraButtons = function () {
    if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
      if (this.i === 1) {
        for (var i = 7; i < 11; i++) {
          $('#button_' + i).remove();
        }
        $('.button_container').prepend('<button id="button_1">1</button>');
        $('#button_1').click(function () {
          var answer = $(this).html();
          trial.addAnswer(answer);
          if ( trial.i === 5 ) {
            trial.trialTextArray[trial.i].hide();
          }
          trial.showTrial();
        });

        $('.common-instruction').text('Press a number from 1 to 6');
      }

      var trialText = $('.trial-text-' + (this.i + 1));

      if (trialText.length) {
        var text = this.i ? 'Press a number from 1 to 6' : 'Press a number from 2 to 10';

        trialText.text(trialText.text().replace(text, ''));
      }
    }
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      this.start( mode );
    }
  };

  Trial.prototype.createData = function() {
    var conditions = [
      {
        jq: $('#condition-a'),
        index: 1
      },
      {
        jq: $('#condition-b'),
        index: 2
      }
    ];
    this.data.push(conditions[0]);
    this.data.push(conditions[1]);
    this.data.shuffle();
  };

  Trial.prototype.pause = function() {
    this.trialTextArray[5].hide();
    this.$trialCounter.hide();
    this.$image.hide();

    this.$pauseText.show();
    this.$textContainer.show();
    this.$textContainer2.show();

    window.onkeypress = this.startKeyListener.bind( this, 'after pause' );
  };

  Trial.prototype.start = function( mode ) {
    if ( mode === 'begin' ) {
      this.$pressInstruction = $('#press_instruction');
      this.$button_container = $('.button_container');
      this.$trialCounterCurrent = $( '.trial-counter .current' );
      this.$trialCounter = $( '.trial-counter' );
      this.trialTextArray = [];
      this.$image = $( '.photo' );
      this.$pauseText = $( '.pause-text' );
      this.$textContainer = $( '.text-container-1' );
      this.$textContainer2 = $( '.text-container-2' );
      this.$container = $( '.application-container' );
      this.$closeButton = $( '.close-button' );
      this.$membersAnswers = $( '.members-answers' );

      if (window.opener.platform.os.family === 'iOS' || window.opener.platform.os.family === 'Android') {
        $('.common-instruction').show();
      }

      for ( var i = 1; i <= 5; i++ ) {
        this.trialTextArray[i] = $( '.trial-text-' + i );
      }
      this.$button_container.show();
    }
    else {
      this.$membersAnswers.show();
    }
    this.$pressInstruction.hide();
    this.$pauseText.hide();
    this.trialTextArray[this.i + 1].show();
    this.$trialCounter.show();
    this.$textContainer.hide();
    this.$textContainer2.hide();

    if (!platform.mobile) {
      window.onkeypress = this.keyListener.bind( this );
    }
    this.showTrial();
  };

  Trial.prototype.saveResult = function () {
    this.$trialCounter.hide();
    this.trialTextArray[5].hide();
    this.$membersAnswers.hide();

    window.opener.player.experiment_window.results=[];
    window.opener.player.experiment_window.results.push(this.data[1].index);

    this.$closeButton.show();
    $('.experiment > p').hide();
    for ( var i = 0; i < 5; i++ ) {
      window.opener.player.experiment_window.results.push(this.answers[i]);
    }
    for (var i = 0; i < 5; i++) {
      setTimeout(function () {
        trial.$closeButton.find('button').focus();
      }, 100);
    }
  };

  Trial.prototype.showTrial = function() {
    if ( this.i < this.trialNum ) {
      this.removeExtraButtons();
      var html = '';
      this.$membersAnswers.html( html );
      this.i++;
      this.$trialCounterCurrent.text( this.i );
      if (this.trialTextArray[this.i - 1]) {
        this.trialTextArray[this.i - 1].hide();
      }
      this.trialTextArray[this.i].show();
      if (this.trialTextArray[this.i - 1]) {
        this.trialTextArray[this.i - 1].removeAttr('aria-label');
      }
      if (window.opener.platform.os.family === 'iOS') {
        this.trialTextArray[this.i].removeAttr('aria-label');
        $('.ada-shadow-speech').html();
        $('.ada-shadow-speech').html('Trial ' + this.i + ' of ' + this.trialNum + this.trialTextArray[this.i].text());
      }
      else {
        this.trialTextArray[this.i].attr('aria-label', 'Trial ' + this.i + ' of ' + this.trialNum + '. ' + this.trialTextArray[this.i].text());
      }

      setTimeout(function () {
        this.trialTextArray[this.i].focus();
      }.bind(this), 0);
    }
    else {
      this.saveResult();
      this.$button_container.hide();
      $('.common-instruction').hide();
      this.$closeButton.find('a').focus();
    }
  };

  Trial.prototype.addAnswer = function ( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function ( event ) {
    var keyCode = event.keyCode || event.which;
    //var mask = this.i === 1 ? '234567890' : '123456';
    var mask = '123456';
    if (this.i === 1) {
      mask = '234567890';
    }
    else if (this.i === 3) {
      mask = '0123456';
    }

    console.log('mask = ' + mask);

    var str;

    str = String.fromCharCode( keyCode );

    if ( mask.indexOf( str ) > -1 ) {
      this.addAnswer( str );
      if ( this.i === 5 ) {
        this.trialTextArray[this.i].hide();
      }
      this.showTrial();
    }
  };

  var trial = new Trial();

  trial.init();
})();
