/**
 * @fileoverview Used to hold a Bar Graph so it can be reused in 
 * the results section
 * 
 * @todo Implement
 */

/**
 * @constructor
 */
var BarGraph = function(container,settings,results){
	if(typeof results.visible == "undefined" || results.visible == true){
		var _CommonAll    = new CommonAll,
			graphSettings = {
	    	// Only animate if we're not using excanvas (not in IE 7 or IE 8)
	        animate: !$.jqplot.use_excanvas,
	        	
	        // The "seriesDefaults" option is an options object that will
	        // be applied to all series in the chart.
	        seriesDefaults:{
	            renderer:$.jqplot.BarRendererC7,
	            rendererOptions: {fillToZero: true,
	    		                  animation: {speed: 1200},
	                              barWidth: 32,
	                              barPadding: 0,
	                              highlightMouseOver: false,
	                              highlightMouseDownr: false,
	                              shadow: false,
	                              shadowDepth: 2,
	                              shadowOffset: 2}
	        },
	        // Show the legend and put it outside the grid, but inside the
	        // plot container, shrinking the grid to accomodate the legend.
	        // A value of "outside" would not shrink the grid and allow
	        // the legend to overflow the container.
	        legend: {
	            show: true,
	            renderer: $.jqplot.EnhancedLegendRendererC7,
	            rendererOptions: {numberRows : 1},
	            placement: 'outsideGraph',
	            location: 'ne' // compass direction, nw, n, ne, e, se, s, sw, w.
	        },
	        axesDefaults: {
	            pad: 0,
	            tickOptions: {
	                showMark: false
	            }
	        },
	        axes: {
	            // Use a category axis on the x axis and use our custom ticks.
	            xaxis: {
	                label: 'Letters used in study',
	                renderer: $.jqplot.CategoryAxisRenderer,
	                borderWidth: 0,
	                ticks: [],
	                width: 10,
	                labelOptions: {
	                    fontFamily: 'Helvetia, Arial, Sans-serif',
	                    fontSize: '12px',
	                    width: 0
	                },
	                drawMajorGridlines: false
	            },
	            // Pad the y axis just a little so bars can get close to, but
	            // not touch, the grid boundaries.  1.2 is the default padding.
	            yaxis: {
	                borderWidth: 0,
	                labelRenderer: $.jqplot.CanvasAxisLabelRenderer,
	                labelOptions: {
	                    fontFamily: 'Helvetia, Arial, Sans-serif',
	                    fontSize: '12px'
	                }
	            }
	        },
	        grid: {
	            shadow: false,
	            borderWidth: 1.0,
	            gridLineColor: '#ededed',
	            background: '#ffffff',
	            borderColor: '#ededed'
	        }
	    };
		
		//if text needs to be swapped in the description swap it
		if(typeof results.swapText != "undefined" && Boolean(results.swapText.length)){
			for(var swap in results.swapText){
				settings.general.description = settings.general.description.replace(results.swapText[swap].swapVar, results.swapText[swap].swapVal);
			}
		}
		
		//build the HTML
		var result   = $('<div/>').addClass("result");
	  	result.append($('<img class="result-aria-image" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAQAAAC1HAwCAAAAC0lEQVR42mNkYAAAAAYAAjCB0C8AAAAASUVORK5CYII=" style="width: 734px; height: 482px; top: 0; left: 0" alt="' + settings.general.title + '. Match stereotypes: your result is ' + Math.round(results.data[0][0]) + ', average result is '  + Math.round(results.data[1][0])  + '. Mismatch stereotypes: your result is ' + Math.round(results.data[0][1]) + ', average result is '  + Math.round(results.data[1][1]) + '.">'));
		var chartDiv = $('<div/>' ).attr({'tabindex':-1, 'aria-hidden': true}).addClass('chartDiv');

		chartDiv.append($('<div class="heading"/>').html(settings.general.title));
		chartDiv.append($('<div/>').attr("id",settings.general.id).addClass('chartContainer'));
		result.append(chartDiv);
	    result.append($('<div/>').attr({'tabindex':0}).addClass('chartDescription').html(settings.general.description));
	    result.append($('<br/>').addClass('clear'));
	    container.append(result);
	    
	    //merge the settings with the graph data
	    var graphSettings = _CommonAll.mergeRecursive(graphSettings,settings.options);
	    
	    //append results data to graph object
	    var axes = ["xaxis","yaxis"];
	    
	    for(var i = 0; i < axes.length; i++){
	    	if(typeof results[axes[i]] != 'undefined'){
	    		for(var axisData in results[axes[i]]){
	    			graphSettings.axes[axes[i]][axisData] = results[axes[i]][axisData];
	    		}
	    	}	
	    }
	    
	    //If data object exists use it, otherwise use results
	    //Prevents experiments from breaking on update
	    if(typeof results.data != 'undefined'){
	    	var graphData = results.data;
	    }
	    else{
	    	var graphData = results;
	    }
	    
	    //Draw the graph 
	    $.jqplot.preDrawSeriesHooks.push(function(currItem){
	    	//if we have a second color do a gradient fill
	    	if(this.seriesColors[1]){
				var ctx = this.canvas._ctx; //canvas 2D renderer

		        var color0 = $.inArray(this.fillColor,this.seriesColors);
		        var color1 = color0+(this.seriesColors.length/2);

              var cw = this._plotDimensions.width;
              var ch = this._plotDimensions.height;

              var canvasPattern = document.createElement("canvas");
              canvasPattern.width = 10;
              canvasPattern.height = 10;
              var contextPattern = canvasPattern.getContext("2d");

              contextPattern.fillStyle = this.seriesColors[color0];
              contextPattern.fillRect(0, 0, 10, 5);
              contextPattern.fillStyle = this.seriesColors[color1];
              contextPattern.fillRect(0, 5, 10, 10);

              var gradient = ctx.createPattern(canvasPattern, 'repeat');

              var canvasPattern1 = document.createElement("canvas");
              var contextPattern1 = canvasPattern1.getContext("2d");
              canvasPattern1.width = this._plotDimensions.width;
              canvasPattern1.height = this._plotDimensions.height;

              contextPattern1.fillStyle = gradient;
              contextPattern1.translate(cw/2, ch/2);
              contextPattern1.rotate((Math.PI/180) * -45);
              contextPattern1.fillRect(-cw, -ch, cw * 2, ch * 2);
              //contextPattern1.setTransform(1, 0, 0, 1, 0, 0);

              var gradient1 = ctx.createPattern(canvasPattern1, 'repeat');

		        this.renderer.shapeRenderer.strokeStyle = gradient1;
		        this.renderer.shapeRenderer.fillStyle = gradient1;
		        this.renderer.shapeRenderer.lineWidth = 1;
		        this.renderer.shapeRenderer.fill = true;
		        this.renderer.shapeRenderer.stroke = true;

	        }
		});
	    var plot = $.jqplot(settings.general.id,graphData,graphSettings);
	}
}