/**
* @fileoverview Used to hold a Bar Graph so it can be reused in
* the results section
*
* @todo Implement
*/

/**
 * @constructor
 */
var LineGraph = function(container,settings,results){
	if(typeof results.visible == "undefined" || results.visible == true){
		var _CommonAll    = new CommonAll,
			graphSettings = {
	        // Only animate if we're not using excanvas (not in IE 7 or IE 8)
	        animate: !$.jqplot.use_excanvas,
	
	        // The "seriesDefaults" option is an options object that will
	        // be applied to all series in the chart.
	        seriesDefaults:{
	    		shadow: false,
	    		lineWidth: 2,
	    		markerOptions: {shadow: false}
	        },
	        
	        // Show the legend and put it outside the grid, but inside the
	        // plot container, shrinking the grid to accomodate the legend.
	        // A value of "outside" would not shrink the grid and allow
	        // the legend to overflow the container.
	        legend: {
	            show: true,
	            renderer: $.jqplot.EnhancedLegendRendererC7,
	            rendererOptions: {numberRows : 1},
	            placement: 'outsideGraph',
	            location: 'ne' // compass direction, nw, n, ne, e, se, s, sw, w.
	        },
	        axesDefaults: {
	            pad: 0,
	            tickOptions: {
	                showMark: false
	            }
	        },
	        axes: {
	            // Use a category axis on the x axis and use our custom ticks.
	            xaxis: {
	                label: 'Letters used in study',
	                renderer: $.jqplot.CategoryAxisRenderer,
	                borderWidth: 0,
	                ticks: [],
	                pad: 0,
	                width: 10,
	                labelOptions: {
	                    fontFamily: 'Helvetia, Arial, Sans-serif',
	                    fontSize: '12px',
	                    width: 0
	                },
	                drawMajorGridlines: false
	            },
	            // Pad the y axis just a little so bars can get close to, but
	            // not touch, the grid boundaries.  1.2 is the default padding.
	            yaxis: {
	                borderWidth: 0,
	                pad: 0,
	                labelRenderer: $.jqplot.CanvasAxisLabelRenderer,
	                labelOptions: {
	                    fontFamily: 'Helvetia, Arial, Sans-serif',
	                    fontSize: '12px'
	                }
	            }
	        },
	        grid: {
	            shadow: false,
	            borderWidth: 1.0,
	            gridLineColor: '#ededed',
	            background: '#ffffff',
	            borderColor: '#ededed'
	        }
	    };
	
		//if text needs to be swapped in the description swap it
		if(typeof results.swapText != "undefined" && Boolean(results.swapText.length)){
			for(var swap in results.swapText){
				settings.general.description = settings.general.description.replace(results.swapText[swap].swapVar, results.swapText[swap].swapVal);
			}
		}
		
		//build the HTML
		var result   = $('<div/>').addClass("result");
		var chartDiv = $('<div/>').attr({'tabindex':0}).addClass('chartDiv');

		chartDiv.append($('<div class="heading"/>').html(settings.general.title));
		chartDiv.append($('<div/>').attr("id",settings.general.id).addClass('chartContainer'));
		result.append(chartDiv);
	    result.append($('<div/>').attr({'tabindex':0}).addClass('chartDescription').html(settings.general.description));
	    result.append($('<br/>').addClass('clear'));
	    container.append(result);
	
	    //merge the settings with the graph data
	    var graphSettings = _CommonAll.mergeRecursive(graphSettings,settings.options);
	    
	    //append results data to graph object
	    var axes = ["xaxis","yaxis"];
	    
	    for(var i = 0; i < axes.length; i++){
	    	if(typeof results[axes[i]] != 'undefined'){
	    		for(var axisData in results[axes[i]]){
	    			graphSettings.axes[axes[i]][axisData] = results[axes[i]][axisData];
	    		}
	    	}	
	    }
	    
	    //If data object exists use it, otherwise use results
	    //Prevents experiments from breaking on update
	    if(typeof results.data != 'undefined'){
	    	var graphData = results.data;
	    }
	    else{
	    	var graphData = results;
	    }
	    
	    //Draw the graph
	    var plot = $.jqplot(settings.general.id,graphData,graphSettings);
	}
}