/**
 * @fileoverview the main function of advanced settings
 * 
 * @author Bo Carlson (bo@chairseven.com)
 */

/**
 * Advanced Settings Main
 * 
 * Closure otherwise events freak out and this jumps to window
 * 
 * You now no longer need to push this up to global using $main
 * or $this while maintaining this. Refer to _Main for this.
 * Please do not send anything to global unless you absolutely have to.
 * You're safe in here.
 * 
 * Anything with a _ can be referenced inside anything inside this but will
 * not be seen from outside. Anything with a var cannot be seen outside of 
 * this. Anything put on this will be seen outside of this.
 * 
 * @param {object} arga (required) - the ARGA wrapper
 * @param {object} xmlParser (required) - The XML Parser
 * @param {jQuery} appContainer (required) - the app container
 * @param {boolean} debug (optional) - set to true for debug info
 * 
 * @constructor
 */
function Main(Arga,XMLParser,appContainer,logging,debug){
	//this
	var _Main = this,
	
	    //give this objects
	    _Arga            = Arga,
	    _XMLParser       = XMLParser,
	    _CommonAll       = new CommonAll,

	    //general
	    _appContainer    = appContainer,
	    _logging         = typeof logging == "boolean" ? logging : false,
	    _debug           = typeof debug == "boolean" ? debug : false;
	
	/**
	 * !!!!START DEBUG CODE!!!!
	 * 
	 * You can load your Arga objects here to make sure the page is setting the values correctly
	 * please remove this when you are done.
	 * 
	 * Also please make sure to update the object in js/arga/arga_wrapper.js before turning your
	 * module in for review. The guys hooking this up need this info.
	 * 
	 */
	//
	if(_debug){
		//call the common debug method
		_CommonAll.debug();
		
		//All of the values for the advanced settings page
		_Arga.setAdvancedSettingsData({text1     : "debug", //string
			 						   select1   : 3, //int
			                           radio1    : 3, //int
			                           checkbox1 : [1,3]}); //int
	}

	/**
	 * !!!!END DEBUG CODE!!!!
	 */

	/**
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!WARNING!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!DO NOT MODIFY ANYTHING BEYOND THIS POINT FOR A      !!!!
	 * !!!!SINGLE MODULE. ALL GRAPH OPTIONS CAN BE CONTROLED   !!!!
	 * !!!!THROUGH XML. ONLY MODIFY THIS CODE IF YOU ARE       !!!!
	 * !!!!WORKING ON THE FRAMEWORK.                           !!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 */
	
	//advanced settings specific data
	var _settings        = _Arga.getAdvancedSettingsData(),
		_userType        = _Arga.getUserType(),
	    _form            = {}; 
	
	//Build the XML
	appContainer.append(_XMLParser.buildHTML(_CommonAll));
    
    //Set the values of the form from ARGA
    setForm(_settings);
    
    //Make it fancy
    //$("form").jqTransform();
    $("form").form();
    
    if(_userType != "3_instructor"){
        var _ARGA = window.opener || window.parent || null;
        // brb: This is keeping students from saving the options
        /*
    	if(!_Arga.isISM()){
    	    disableForm();
    	}
        */
    	$('#indyStudy').toggleClass('hidden');
    }
    
    //save routine
    $("form").submit({settings:_settings,Arga:_Arga},function(e){
    	e.preventDefault();
    	var settings = e.data.settings;

        // brb: save off faces/words values
        window.parent.player.EXP_FACES = Number($("#faces").val());
        window.parent.player.EXP_WORDS = Number($("#words").val());

        console.log("EXP_FACES = " + window.parent.player.EXP_FACES);
        console.log("EXP_WORDS = " + window.parent.player.EXP_WORDS);
        
    	//get the settings out of the form
    	for(setting in settings){
    		var currInput = $('[name='+setting+']');
    		if(currInput.length == 1){
    			settings[setting] = currInput.val();
    		}
    		else if(currInput.length > 1){
    			if(currInput.attr('type') == "checkbox"){
    				settings[setting] = [];
    			}

    			currInput.each(function(){
        			if(this.checked == true){
        				if($(this).attr('type') == "checkbox"){
        					if(jQuery.isArray(settings[setting])){
        						settings[setting].push(this.value);
        					}
        					else{
        						settings[setting] = this.value;
        					}
        				}
        				else{
        					settings[setting] = this.value;
        				}
        			}
        		});
    		}
    		else{
    			settings[setting] = "false";
    		}
    	}
      var quiz1 = window.top.$('#quiz1');
      var quiz2 = window.top.$('#quiz2');
      var quiz3 = window.top.$('#quiz3');
      var quiz4 = window.top.$('#quiz4');
      var quiz5 = window.top.$('#quiz5');
      quiz3 = window.top.$('#quiz3' ).find('[data-type="number"]');
      quiz4 = window.top.$('#quiz4' ).find('[data-type="number"]');
      quiz5 = window.top.$('#quiz5' ).find('[data-type="number"]');
      quiz3.text('1.');
      quiz4.text('2.');
      quiz5.text('3.');
      quiz1.hide();
      quiz2.hide();
      var savedNode = document.querySelector('#saved');
      savedNode.setAttribute('style', 'display:block; color: darkblue !important; position: absolute; margin-top: -2em;');
      //setTimeout("document.querySelector('#saved').setAttribute('style', 'display:none')",2000);
    	//log to console
    	if(logging)
    		_CommonAll.log(settings);
        //Save and get out
    	var result = e.data.Arga.setAdvancedSettingsData(settings);
        //only disable if we're not in ISM
    	if(!result){
          $('select').attr({"disabled": "disabled", "aria-disabled": "true"});
          $('.rowElem').css({"color": "#888"});
          $('.ui-state-default').css({"color": "#888"});
          $('#submit').css({"color": "#555555","cursor":"default"}).attr({"disabled": "disabled", "aria-disabled": "true"});

          if (platform.android) {
            $('select').each(function() {
              var $this = $(this);
              var value = $this.val();
              var label = $this.prev('label').text().trim();
              var text = $this.find('option[value=' + value + ']').text().trim();
              $this.attr({'aria-hidden': 'true', 'tabindex': '-1'});
              $this.parent('.rowElem').append($('<span class="text" role="combobox" tabindex="0">' + label + ' ' + text + '. Combobox, dimmed</span>'));
            })
          }

          window.parent.document.getElementById('digfir_section_1').querySelector('.next_button:not(.prev_button)').focus();
        }
    	
    	return false;
    });
    
    //indy study event listener
    $('#indyStudy').on('click',function(e,ui){
    	indyStudy($('input#indyStudy').attr('checked'));
    });
    
    /**
     * Sets the form values based on data in ARGA
     */
    function setForm(settings){
    	for(var setting in settings){
        	var currInput = $('[name='+setting+']');
        	
        	//If the length is 1 we know it's not check boxes or radios
        	if(currInput.length == 1){
        		currInput.val(settings[setting]);
        	}
        	else{
        		var i = 0;
        		currInput.each(function(){
        			var tmpSetting;
                    if ($.isArray(settings[setting])) {
                        tmpSetting = settings[setting][i];
                    } else {
                        tmpSetting = settings[setting];
                    }
        			
        			if(this.value == tmpSetting){
        				$(this).attr("checked", "checked");
        				i++;
        			}
        			else{
        				$(this).attr("checked", false);
        			}
        		});
        	}
    	}

      if (platform.iOS) {
	  	$('label').attr('tabindex', '-1');
        $('select').change(function (evt) {
          var target = evt.target;
          setTimeout(function () {
            $(target.previousSibling).focus();
          }, 1000);
          setTimeout(function () {
            $(target).focus();
          }, 1500);
        });
      }

      if (navigator.userAgent.match(/Android 8/i)) {
        $('select').change(function (evt) {
          $(target).focus();
        });
	  }

      if (platform.android) {
        $( 'input[type="submit"]' ).replaceWith( '<button id="submit" class="submit_btn"><span>save settings</span></button>' );
	  }

    }
    function errorDialog(text){
        return $('<div></div>').html(text)
            .dialog({autoOpen: false,
                title: 'Error',
                resizable: false,
                modal: true,
                draggable: false,
                buttons: { "OK": function(){$(this).dialog("close");}}});
    }
    function disableForm(){
			var blinds = $('.rowElem');

    	$('#description').toggleClass('hidden');
    	$('#disabledText').toggleClass('hidden');

			// $('select').attr({"disabled": "disabled", "aria-disabled": "true"});

			for(var blind = 0; blind < blinds.length; blind++){
				var currBlind = $(blinds[blind]);
				currBlind.append($('<div class="elemCover"/>').css({
					"z-index": 10,
					"width": currBlind.width(),
					"height": currBlind.height(),
					"position": "absolute",
					"top": 0,
					"left": 0
				}));
			}

			blinds.css({"color": "#888"});
			$('.ui-state-default').css({"color": "#888"});
			$('#submit').css({"color": "#555555","cursor":"default"}).attr({"disabled": "disabled", "aria-disabled": "true"});

      if (platform.android) {
        $('select').each(function() {
          var $this = $(this);
          var value = $this.val();
          var text = $this.children('option[value=' + value + ']').text().trim();
          var label = $this.prev('label').text().trim();
          var text = $this.find('option[value=' + value + ']').text().trim();
          $this.attr({'aria-hidden': 'true', 'tabindex': '-1'});
          $this.parent('.rowElem').append($('<span class="text" role="combobox" tabindex="0">' + label + ' ' + text + '. Combobox, dimmed</span>'));
        })
      }
    }
    function enableForm(){
    	$('#description').toggleClass('hidden');
    	$('#disabledText').toggleClass('hidden');
			$('.elemCover').remove();
    	$('.rowElem').css({"color":"#363636"});
    	$('.ui-state-default').css({"color": "#363636"});
			$('#submit').css({"color": "#FFF","cursor":"pointer"}).removeAttr("disabled aria-disabled");
			if (platform.android) {
				$('select').removeAttr('aria-hidden tabindex');
				$('.text').remove();
			}
    }

    function indyStudy(checked){
    	if(typeof checked !== 'undefined'){
    	    enableForm();
    	    _settings.indyStudy = "on";
    	}
    	else{
    	    disableForm();
    	    _settings.indyStudy = "off";
    	    _Arga.clearIndyStudyData();
    	}
    	_Arga.setAdvancedSettingsData(_settings);
    }
};
