/**
 * @fileoverview the main of the results
 * 
 * @author Bo Carlson (bo@chairseven.com)
 */

/**
 * Results Main
 * 
 * Closure otherwise events freak out and this jumps to window
 * 
 * You now no longer need to push this up to global using $main
 * or $this while maintaining this. Refer to _Main for this.
 * Please do not send anything to global unless you absolutely have to.
 * You're safe in here.
 * 
 * Anything with a _ can be referenced inside anything inside this but will
 * not be seen from outside. Anything with a var cannot be seen outside of 
 * this. Anything put on this will be seen outside of this.
 * 
 * @param {object} arga (required) - the ARGA wrapper
 * @param {object} xmlParser (required) - The XML Parser
 * @param {jQuery} appContainer (required) - the app container
 * @param {boolean} debug (optional) - set to true for debug info
 * 
 * @constructor
 */
function Main(Arga,XMLParser,appContainer,logging,debug){
	//this
	var _Main = this,
	
	    //give this objects
	    _Arga            = Arga,
	    _XMLParser       = XMLParser,
	    _CommonAll       = new CommonAll,
	    
	    //general
	    _appContainer    = appContainer,
	    _logging         = typeof logging == "boolean" ? logging : false,
        _debug           = typeof debug == "boolean" ? debug : false;
	
	/**
	 * !!!!START DEBUG CODE!!!!
	 * 
	 * You can load your Arga objects here to make sure the page is setting the values correctly
	 * please remove this when you are done.
	 * 
	 * Also please make sure to update the object in js/arga/arga_wrapper.js before turning your
	 * module in for review. The people hooking this up need this info.
	 */
	if(_debug){
		//call the common debug method
		_CommonAll.debug();
		
		//All of the values for the results page
		_Arga.setResultsData([{data: [[30,20],[20,10]],
			                   xaxis: {ticks: ["Letter in Word","Letter Alone"]},
			                   yaxis: {tickInterval: 10,
			                	       min: 0,
			                	       max: 40}
			                   }]);
	}
	/**
	 * !!!!END DEBUG CODE!!!!
	 */

	/**
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!WARNING!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 * !!!!DO NOT MODIFY ANYTHING BEYOND THIS POINT FOR A      !!!!
	 * !!!!SINGLE MODULE. ALL GRAPH OPTIONS CAN BE CONTROLED   !!!!
	 * !!!!THROUGH XML. ONLY MODIFY THIS CODE IF YOU ARE       !!!!
	 * !!!!WORKING ON THE FRAMEWORK.                           !!!!
	 * !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	 */
	
	//results specific data
	var _experimentState    = _Arga.getExperimentData(),
		_experimentSettings = _Arga.getAdvancedSettingsData(),
	    _resultsData        = _Arga.getResultsData(),
	    _resultsCSV         = _Arga.getResultsCSV(),
        _classResultsCSV    = _Arga.getClassResultsCSV(),
	    _graphData          = _XMLParser.buildDataObject();
    
    //Initialize the Graphs factories
	for(var i = 0; i < _graphData.length; i++){
		new Graph(_appContainer,_graphData[i],_resultsData[i]);
	}
	
	_appContainer.append('<div id="downloadBtnHolder"/>');
    $('#downloadBtnHolder').append($('<button id="downloadBtn" data-clipvar="participant">').text("Download My Data"));
    $('#downloadBtnHolder').append('&nbsp;&nbsp;&nbsp;');
    $('#downloadBtnHolder').append($('<button id="downloadClassBtn" data-clipvar="class">').text("Download Class Data"));
    
    $(document).ready(function() {
        $('#downloadBtn').on('click', function() {
            $('#download_form textarea[name=data]').val(_resultsCSV);
            $('#download_form input[name=filename]').val('implicit_association_test_data.csv');
            $('#download_form form').submit();
        });

        $('#downloadClassBtn').on('click', function() {
            $('#download_form textarea[name=data]').val(_classResultsCSV);
            $('#download_form input[name=filename]').val('implicit_association_test_class_data.csv');
            $('#download_form form').submit();
        });

        // brb: Don't let students have class data
        if (_Arga.getUserType() != '3_instructor') {
            $("#downloadClassBtn").remove();
        }
        
    });
};
