/**
 * @fileoverview Handles loading of results datasets from results.xml
 * Called from factory.js
 * 
 * @author 
 * 
 * @todo Possibly get this running of a base object for common methods like setData 
 */

/**
 * Set us up the ResultsParser object
 * 
 * @param {object} data (required)      - XML data to set
 * @param {string} container (optional) - ID of the container for html output 
 * @constructor
 */
var ResultsParser = function(data,container){
	this._data       = undefined;
	this._dataObject = {};
	this._xml        = undefined;
	this._container  = 'results';
	
	//Set the data
	this.setData(data);
	
	//Set the container
	this.setContainer(container);
};

/**
 * Load the object with a new XML object
 * @todo Make this part of a base object
 * 
 * @param {object} data (required) - New XML data to set
 */
ResultsParser.prototype.setData = function(data){
	//Set data and xml object
	if(data){
		this._data = data;
		this._xml  = $(this._data);
	}
	else{
		//@todo exception on failure
		alert('ResultsParser.setData needs data');
	}
};

/**
 * Set the html container for drawing HTML from XML
 * @todo Make this part of a base object
 * 
 * @param {string} container (required) - ID of the container
 */
ResultsParser.prototype.setContainer = function(container){
	/**
	 * set up the container for HTML output. If it doesn't exist we use the
	 * default for the document
	 */ 
	if(container && typeof container === "string"){
		this._container = $('#'+container);
	}
	else{
		//@todo Could throw an exception here
		this._container = $('#'+this._container);
	}
};

/**
 * Build and display html from the XML data
 */
ResultsParser.prototype.buildHTML = function(){
};

/**
 * Build a data object from values in XML file
 * Very useful for large data sets
 */
ResultsParser.prototype.buildDataObject = function(){
	var tmpDataObject = this._xml.children('results').children('graph');
	this._dataObject = [];
	
	//loop the data elements
	for(var i = 0; i <  tmpDataObject.length; i++){
		var currData = $(tmpDataObject[i]);
		var tmpDataObjectChildren = currData.children();
		this._dataObject.push({});
		
		//Now loop all sets and items
		for(var j = 0; j <  tmpDataObjectChildren.length; j++){
			this._dataObject[i][tmpDataObjectChildren[j].tagName] = {};
			crawlDataXML(tmpDataObjectChildren[j],this._dataObject[i]);
		}
	}

	//return the data through the function param
	return this._dataObject;
	
	//recursive function to crawl the data xml
	function crawlDataXML(DataXML,parentObj,isArray){
		//make it a jQuery object
		DataXML = $(DataXML);
		
		for(var i = 0; i < DataXML.length; i++){
			var tag = DataXML[i].tagName;
			
			if($(DataXML[i]).children().length){
				if(Boolean($(DataXML[i]).attr("array"))){
					parentObj = parentObj[tag] = [];
				}
				else if(isArray && tag == "object"){
					parentObj.push({});
					parentObj = parentObj[parentObj.length-1];
				}
				else{
					parentObj = parentObj[tag] = {};
				}
				for(var j = 0; j < $(DataXML[i]).children().length; j++){
					crawlDataXML($(DataXML[i]).children()[j],parentObj,$(DataXML[i]).attr("array"));
				}
			}
			else{
				var tmpItem = $(DataXML[i]);
				
				if(isArray){
					switch(tmpItem.attr('type')){
					case 'int':
						parentObj.push(parseInt(tmpItem.text()));
						break;
					case 'float':
						parentObj.push(parseFloat(tmpItem.text()));
						break;
					default:
						parentObj.push($.trim(tmpItem.text()));
						break;
					}
				}
				else{
					switch(tmpItem.attr('type')){
					case 'int':
						parentObj[tag] = parseInt(tmpItem.text());
						break;
					case 'float':
						parentObj[tag] = parseFloat(tmpItem.text());
						break;
					default:
						parentObj[tag] = $.trim(tmpItem.text());
						break;
					}
						
				}
			}
		}
	}
};