(function() {
  'use strict';

  var alts = { a: [, 'An open MacBook with a color screen', 'An iPhone', 'A blue Apple watch with a screen displaying different application icons.',
                     'A bright red modern convertible car.', 'A silver and black blender with a large dial.', 'A colorful playground with plastic slides, swing set, and other items.',
                     'A gas pump with built in payment options.', 'The New York skyline with the One World Trade Center building.', 'A parking meter with a credit card payment option',
                     'A woman listening to headphones plugged into a mp3 player.', 'Red ski boots clipped into sleek black skis',
                     'A trash can with a door on the top side that also compacts trash.', 'A pink and white clothing iron with a water compartment to steam clothes.',
                     'A modern plastic high chair.', 'A stroller with three wheels and safety features', 'A large flat screen television', 'An all glass elevator going up a high rise building.',
                     'A high speed electric train', 'A three wheel blue scooter made of plastic and metal', 'An inline roller scate' ],
    b: [, 'An open laptop with a black screen', 'A cell phone that flips open and has an antenna.', 'A gray watch with digital numbers.', 'An older dark red convertible car.',
          'A yellow blender with push buttons.', 'A metal merry-go-round in an open park.', 'A gas pump that only displays the price.',
          'The New York skyline with the original two World Trade Center buildings.', 'A metal parking meter that only takes coins',
          'A woman listening to headphones plugged into a CD player.', 'Wooden skis attached to lace-up ski boots.', 'A beat up metal trash can with no lid, attached to a pole.',
          'A yellow clothing iron with no steam option.', 'An old plastic high chair', 'An antiques stroller with large wheels',
          'A small television with two built in dials on a television stand', 'An elevator with an iron gated door', 'A steam locomotive',
          'A red wooden scooter with two wheels', 'A pair of clip-on roller skates'] };

  var dataNames = [ 'Richard', 'Tony', 'Emma', 'Liam', 'Sophia', 'Daniel', 'William', 'John', 'James', 'Benjamin', 'Robert', 'Eric',
    'Patrick', 'Caroline', 'Jennifer', 'Anna', 'Julia', 'Allison', 'Natalie', 'Jessica', 'Michelle', 'Jenna' ];

  Array.prototype.shuffle = function() {
    for ( var i = this.length - 1; i > 0; i-- ) {
      var num = Math.floor( Math.random() * ( i + 1 ) );
      var d = this[ num ];
      this[ num ] = this[ i ];
      this[ i ] = d;
    }
    return this;
  };

  var Trial = function() {
    this.trialNum = 30;
    this.i = 0;
    this.data = [];
    this.answers = [];
    this.names = [];
    this.mapQuestions = [];
    this.results = {
      'practice': 0,
      'unanimous': 0,
      'dissenter': 0
    };
  };

  Trial.prototype.init = function() {
    this.$trialCounterCurrent = $( '.trial-counter .current' );
    this.$trialCounter = $( '.trial-counter' );
    this.$trialText = $( '.trial-text' );
    this.$image = $( '.photo' );
    this.$pauseText = $( '.pause-text' );
    this.$textContainer = $( '.text-container' );
    this.$container = $( '.application-container' );
    this.$closeButton = $( '.close-button' );
    this.$membersAnswers = $( '.members-answers' );

    this.$textContainer.show();
    this.$textContainer.focus();
    this.createData();
    dataNames.shuffle();
    this.names = dataNames.slice( 0, 5 );
    this.platform = window.opener.platform;
    this.isIos = window.opener.platform.os.family === 'iOS';
    this.isAndroid = window.opener.platform.os.family === 'Android';

    // var live = this.platform.os.family === 'OS X' ? 'polite' : 'assertive';

    $('body').prepend('<div class="ada-shadow-speech" aria-live="assertive"></div>');

    this.$liveRegion = $('.ada-shadow-speech');

    if (this.platform.os.family === 'iOS') {
      $('*:not(button)').attr('tabindex', 0);
      $('*').removeAttr('aria-label');
      $('body').prepend($('<span id="focus_elem" tabindex="0" class="visually-hidden">Experiment Tab</span>'));
      setTimeout(function () {
        $('#focus_elem').focus();
      }, 200);
      this.$liveRegion.attr('aria-live', 'polite');
    }
    if (this.platform.os.family === 'iOS' || this.platform.os.family === 'Android') {
      $('.press_instruction').html('Press the space bar or <span class="visually-hidden">double</span> tap<br/>to begin the experiment.');

      if (this.platform.os.family === 'Android') {
        $('.press_instruction').attr('aria-label', $('.press_instruction').attr('aria-label').replace('Press the space bar', 'Press the space bar or double tap'));
      }
      $('.experiment').bind('click', function () {
        trial.start('begin');
        $('.experiment').unbind('click');
      });
      this.$trialText.text(this.$trialText.text().replace(/the “y” key/g, 'the “Yes” button').replace(/the “n” key/g, 'the “No” button'));
      this.$trialText.attr('aria-label', this.$trialText.text().replace(/the “y” key/g, 'the “Yes” button').replace(/the “n” key/g, 'the “No” button'));
    }

    var buttonsStr = '';
    for (var i = 0; i < 2; i++) {
      var text = i ? 'No' : 'Yes';
      buttonsStr += '<button id="button_' + i + '">' + text + '</button>';
    }

    $('.experiment').append($('<div class="button_container">' + buttonsStr + '</div>'));

    this.$buttonContainer = $('.button_container');
    this.$buttonContainer.find('button').each(function (i, btn) {
      $(btn).click(function () {
        trial.addAnswer(!i);
        if ( trial.i === 10 ) {
          trial.pause();
        }
        else if ( trial.i >= 10 ) {
          trial.showTrialSecond();
        }
        else {
          trial.showTrial();
        }
      });
    });
    this.$buttonContainer.hide();
    document.onkeypress = this.startKeyListener.bind( this, 'begin' );
  };

  Trial.prototype.startKeyListener = function( mode, evt ) {
    var keyCode = evt.keyCode || evt.which;
    console.log( keyCode );
    var str = String.fromCharCode( keyCode );
    if ( str === ' ' ) {
      this.start( mode );
    }
  };

  Trial.prototype.createData = function() {
    var i, j;
    for ( i = 1; i <= 20; i++ ) {
      this.data.push({ 'src': 'images/soel-' + i + 'a.jpg', 'new': true, className: i + 'a', 'alt': alts.a[i] });
      this.data.push({ 'src': 'images/soel-' + i + 'b.jpg', 'new': false, className: i + 'b', 'alt': alts.b[i] });
    }
    for ( i = 0; i < 2; i++ ) {
      for ( j = 0; j < 10; j++ ) {
        this.mapQuestions.push( i );
      }
    }
    this.mapQuestions.shuffle();
    this.data.shuffle();
  };

  Trial.prototype.pause = function() {
    this.$trialText.hide();
    this.$trialCounter.hide();
    this.$image.hide();
    this.$buttonContainer.hide();
    this.$liveRegion.html('');

    this.$pauseText.css('height', '160px').append(this.$textContainer);
    this.$pauseText.show();
    this.$textContainer.removeAttr('tabindex').css({'position': 'static', 'width': '100%'}).show();

    this.$pauseText.focus();
    if (this.platform.os.family === 'iOS' || this.platform.os.family === 'OS X' || this.platform.os.family === 'Android') {
      $('.exp2').bind('click', function () {
        trial.start('after pause');
        $('.exp2').unbind('click');
      });

      if (this.isIos) {
        this.setAriaFocusTo(this.$pauseText, 0);
      }
      if (this.isAndroid) {
        this.$pauseText.attr('aria-label', this.$pauseText.attr('aria-label').replace('Press the space bar', 'Press the space bar or double tap'));
      }
    }
    document.onkeypress = this.startKeyListener.bind( this, 'after pause' );
  };

  Trial.prototype.start = function( mode ) {
    if ( mode !== 'begin' ) {
      this.$membersAnswers.show();
      this.$pauseText.hide();
      this.$trialText.show();
      this.$trialCounter.show();

      this.$textContainer.hide();

      this.showTrialSecond();
    }
    else {
      this.$pauseText.hide();
      this.$trialText.show();
      this.$trialCounter.show();
      this.$textContainer.hide();

      document.onkeypress = this.keyListener.bind( this, 'begin' );
      this.showTrial();
    }

    setTimeout(function() {
      this.$trialText.focus();
      if (this.isIos) {
        this.$trialText.find('[tabindex="0"]').eq(0).focus();
      }
    }.bind(this), 10);
  };

  Trial.prototype.saveResult = function() {
    this.$trialCounter.hide();
    this.$trialText.hide();
    this.$image.hide();
    this.$membersAnswers.hide();
    this.$buttonContainer.hide();

    this.$closeButton.show();
    this.$closeButton.find('button').focus();

    var i;
    this.results = {
      'practice': 0,
      'unanimous': 0,
      'dissenter': 0
    };

    for ( i = 0; i < 10; i++ ) {
      if ( this.data[ i ].new === this.answers[ i ] ) {
        this.results[ 'practice' ]++;
      }
    }
    for ( i = 10; i < this.trialNum; i++ ) {
      if ( this.data[ i ].new === this.answers[ i ] ) {
        if ( this.mapQuestions[ i - 10 ] ) {
          this.results[ 'dissenter' ]++;
        }
        else {
          this.results[ 'unanimous' ]++;
        }
      }
    }

    window.results = this.results;
  };

  Trial.prototype.showTrial = function() {
    this.$image.show();
    this.$image.find( 'a' ).hide();
    var curImg = this.$image.find( '#' + this.data[ this.i ].className );
    curImg.show().focus();

    var alt = this.$image.find( '#' + this.data[ this.i ].className ).find('img').attr('alt');
    var ieAlt = this.i === 0 ? this.$image.find( '#' + this.data[ this.i ].className ).find('img').attr('alt') : '';
    var trialText = this.i === 0 ? $('.trial-text').text() : '';

    this.i++;
    this.$trialCounterCurrent.text( this.i );
    if (this.platform.os.family === 'iOS' || this.platform.os.family === 'OS X') {
      this.$liveRegion.html(this.$trialCounter.text() + ' ' + trialText + ' ' + alt);
      this.platform.os.family === 'OS X' ? this.$buttonContainer.hide() : this.$buttonContainer.show();
    } else if (this.platform.os.family === 'Android') {
      setTimeout(function() {
        this.i === 0 && curImg.focus();
        this.$liveRegion.html(this.$trialCounter.text() + ' ' + trialText + ' ' + alt);
      }.bind(this), 100);
      this.$buttonContainer.show();
    } else {
      this.$liveRegion.html(this.$trialCounter.text() + ' ' + ieAlt);
    }
  };

  Trial.prototype.showTrialSecond = function() {
    if ( this.i < this.trialNum ) {
      this.$buttonContainer.hide();
      if (this.platform.os.family === 'iOS' || this.platform.os.family === 'OS X' || this.platform.os.family === 'Android') {
        this.$membersAnswers.show();
        this.$membersAnswers.css( { position: 'static', float: 'left', 'margin-left': '50px' } );
      } else {
        this.$membersAnswers.css( { left: 365 } );
      }

      document.onkeypress = function() {};

      this.$image.show();
      this.$image.find( 'a' ).hide();

      var trialText = this.i === 10 ? $('.trial-text').text() : '';
      var n = ( this.mapQuestions[ this.i - 10 ] ) ? Math.floor( Math.random() * 5 ) : 6;
      var html = '<div class="members-text">This is how your fellow classmates rated this image:</div>',
        ariaLabel = 'This is how your fellow classmates rated this image. ';
      for ( var i = 0; i < this.names.length; i++ ) {
        var answer = ( this.data[ this.i ].new ) ? 'no' : 'yes';
        if ( n === i ) {
          answer = ( this.data[ this.i ].new ) ? 'yes' : 'no';
        }
        ariaLabel += this.names[i] + ' ' + answer + '. ';
        html += '<p>' + this.names[i] + ': ' + answer + '</p>';
      }
      this.$membersAnswers.html( html );
      this.$membersAnswers.attr( {'aria-label' : ariaLabel, tabindex: '0' } );
      this.i++;
      this.$trialCounterCurrent.text( this.i );

      this.$image.find( '#' + this.data[ this.i - 1 ].className ).show();
      var alt = this.$image.find( '#' + this.data[ this.i - 1 ].className ).find('img').attr('alt');
      document.onkeypress = this.keyListener.bind( this, 'after pause' );
      this.$membersAnswers.css( { left: -70 } );
      if (this.platform.name === 'IE') {

      }
      else {
        if (!this.isAndroid) {
          this.$membersAnswers.attr({'aria-describedby': this.data[ this.i - 1 ].className});
        }
      }
      if (this.platform.os.family === 'iOS' || this.platform.os.family === 'Android') {
        this.$buttonContainer.show();
      }

      var liveText = this.platform.name === 'IE' || this.platform.os.family === 'OS X'
        ? this.$trialCounter.text() + ' ' + ((this.platform.os.family === 'OS X' || this.platform.os.family === 'iOS') && this.i === 11 ? trialText : '' ) + ' ' + (this.i === 11 ? ariaLabel : '') + alt : this.$trialCounter.text() + ' ' + (this.i === 11 ? ariaLabel + ' ' + alt : '');
      this.$liveRegion.html(liveText);
      if (this.platform.os.family === 'iOS') {
        this.$membersAnswers.removeAttr('aria-label').removeAttr('tabindex');
        this.setAriaFocusTo(this.$membersAnswers);
        liveText = this.$trialCounter.text() + ' ' + (this.i === 11 ? trialText : '' ) + ' ' + ariaLabel + ' ' + alt;
        this.$liveRegion.html(liveText);
      }
      else if (this.isAndroid) {
        var delay = this.isAndroid ? 350 : 550;
        this.setAriaFocusTo(this.$membersAnswers, 0);
        setTimeout(function () {
          liveText = this.$trialCounter.text() + ' ' + (this.i === 11 ? trialText : '' ) + ' ' + ariaLabel + ' ' + alt;
          this.$liveRegion.html(liveText);
        }.bind(this), delay);
      }
      else {
        $('.experiment').focus();
        setTimeout(function() {
          this.$membersAnswers.focus();
        }.bind(this), 10);
      }
    }
    else {
      this.saveResult();
    }
  };

  Trial.prototype.addAnswer = function( answer ) {
    this.answers.push( answer );
  };

  Trial.prototype.keyListener = function( mode, event ) {
    var keyCode = event.keyCode || event.which;
    var str;

    str = String.fromCharCode( keyCode );

    if ( 'yYnN'.indexOf( str ) > -1 ) {
      ( str === 'y' || str === 'Y' ) && ( this.addAnswer( true ) );
      ( str === 'n' || str === 'N' ) && ( this.addAnswer( false ) );

      if ( this.i === 10 ) {
        this.pause();
      }
      else if ( mode !== 'begin' ) {
        this.showTrialSecond();
      }
      else {
        this.showTrial();
      }
    }
  };
  Trial.prototype.setAriaFocusTo = function ($elem, delay) {
    delay = delay || 0;

    setTimeout(function () {
      $elem = typeof $elem === 'string' ? $($elem) : $elem;

      if (this.isIos && $elem.find('*').length) {
        $elem.find('.ios-vo-focus-fix').remove();
        // $elem.find('.visible-part').remove();

        $elem.css('position', 'relative');
        var $elemText = $elem.attr('aria-label') || $elem.text();
        var itemDescribedByText = $('#' + $elem.attr('aria-describedby')).text() || '';
        var $iosVoFocusableElem = $('<div class="ios-vo-focus-fix" tabindex="0">' + $elemText + ' ' + itemDescribedByText + '</div>');
        if (!$elem.find('.visible-part').length) {
          $elem.html('<div aria-hidden="true" class="visible-part">' + $elem.html() + '</div>');
        }
        $elem.prepend($iosVoFocusableElem);
        $elem = $iosVoFocusableElem;
      }

      $elem.focus();
    }.bind(this), delay);
  };

  var trial = new Trial();
  var resourcesLoaded = 0;
  var images = [];
  for ( var i = 1; i <= 20; i++ ) {
    images.push({src: 'images/soel-' + i + 'a.jpg', className: i + 'a', 'alt': alts.a[i]});
    images.push({src: 'images/soel-' + i + 'b.jpg', className: i + 'b', 'alt': alts.b[i]});
  }

  var onEnd = function () {
    resourcesLoaded++;

    if ( resourcesLoaded === images.length ) {
      setTimeout( function () {
        $( '.preloader' ).hide();
        $( '.text-preloader' ).hide();
        trial.init();
      }, 1000 );
    }
  };

  var x = 0,
    minX = 0,
    maxX = 350,
    step = Math.ceil( ( maxX - minX ) / images.length );


  var stepAnim = function( i ) {
    i++;
    if ( i <= step ) {
      setTimeout( function() {
        x++;
        if ( x <= ( maxX - minX ) ) {
          $( '.loading' ).css( 'width', x );
          stepAnim( i );
        }
      }, 1000 / step );
    }
  };

  images.forEach( function( image ) {
    var img = new Image();

    img.onload = function() {
      stepAnim( 0 );
      onEnd();
      var $img = $('<a href="' + image.src + '" target="_blank"></a>')
        .append( $( img ) )
        .hide()
        .attr({'tabindex': 0, 'id': image.className});
      $( '.photo' ).append( $img );
    };

    img.alt = image.alt;
    img.src = image.src;
  } );
})();
